import os
import json

from gbdx_cloud_harness.test.conftest import my_vcr
from gbdx_cloud_harness.workflow import Workflow


def test_json_build(TestApp, mock_auth):
    TestApp.task.run_name = 'Workflow_test'
    wf_obj = Workflow(TestApp.task, auth=mock_auth)
    wf_json = wf_obj._build_worklfow_json()

    tasks = wf_json['tasks']

    assert len(tasks) == 2

    custom_task_name = None
    custom_task_output_names = None

    for task in tasks:

        if task['taskType'] == 'MyCustomTask':
            inputs = task['inputs']
            outputs = task['outputs']
            assert len(inputs), 3
            assert len(outputs), 1
            custom_task_name = task['name']
            custom_task_output_names = [p['name'] for p in task['outputs']]
        elif task['taskType'] == 'StageDataToS3':
            inputs = task['inputs']
            assert len(inputs), 2

            parsed_s3_loc = [x['value'].split('/')[2:] for x in inputs if x['name'] == 'destination'][0]
            exp_names = ['not_provided', 'not_provided', 'Workflow_test', 'output_port']
            assert exp_names == parsed_s3_loc

            parsed_source = [x['source'].split(':') for x in inputs if x['name'] == 'data'][0]
            assert parsed_source[0] == custom_task_name
            assert True if parsed_source[1] in custom_task_output_names else False
        else:
            assert False  # Fail test


@my_vcr.use_cassette('gbdx_cloud_harness/test/vcr_cassettes/workflow.yaml', filter_headers=['authorization'])
def test_execute_workflow(TestApp, test_path, mock_auth):

    test_wf_file = os.path.join(test_path, 'test_valid_wf.json')

    vcr_filename = os.path.join(
        test_path, 'vcr_cassettes', 'workflow.yaml'
    )

    if os.path.isfile(vcr_filename):
        wf_obj = Workflow(TestApp.task, auth=mock_auth)
    else:
        wf_obj = Workflow(TestApp.task)

    with open(test_wf_file, 'r') as f:
        wf_json = f.read()
        print(wf_json)
        wf_obj.execute(override_wf_json=json.loads(wf_json))

    assert wf_obj.id is not None
    assert isinstance(int(wf_obj.id), int)
    assert wf_obj.status['state'] == 'pending'
    assert wf_obj.status['event'] == 'submitted'

    assert wf_obj.succeeded is False
    assert wf_obj.complete is False

    wf_obj.id = '4392617339332738708'  # A completed workflow
    wf_obj._refresh_status()

    assert wf_obj.succeeded is True
    assert wf_obj.complete is True



import json
import os
from gbdx_task_template.test.test_base import TestBase


class TestTask(TestBase):

    def test_task_json(self):
        print('=====', os.environ.get('REMOTE_WORK_PATH'))
        task_dict = json.loads(self.task.json())

        self.assertEqual(task_dict['name'], 'MyCustomTask')

        # Check Ports
        self.assertEqual(len(task_dict['inputPortDescriptors']), 3)
        for port in task_dict['inputPortDescriptors']:
            self.assertIn(port['name'], self.basic_inport_names)

        self.assertEqual(len(task_dict['outputPortDescriptors']), 1)
        for port in task_dict['outputPortDescriptors']:
            self.assertIn(port['name'], self.basic_outport_names)

        # Check Properties
        task_properties = task_dict['properties']
        self.assertEqual(task_properties['timeout'], 9600)
        self.assertEqual(task_properties['isPublic'], False)

    def test_task_validate(self):
        print('=====', os.environ.get('REMOTE_WORK_PATH'))
        self.assertTrue(self.task.is_valid())



import os
import sys

if sys.version_info[0] == 3:
    from urllib.parse import urlparse
else:
    # Python 2 import
    from urlparse import urlparse


class LocalPortValidationError(Exception):
    pass


class RemotePortValidationError(Exception):
    pass


class Port(object):
    """
    A Class to configure Port instances for the custom application
    """

    def __init__(self, port_type, direction, required, value):
        # Flag as input or output port
        self.__direction = direction
        self.__task_json = dict()

        # Load Task JSON attrs
        self.__task_json['name'] = None
        self.__task_json['type'] = port_type
        self.__task_json['required'] = required
        self.__task_json['description'] = 'cloud-harness'

        if port_type == 'directory':
            # For directory ports, there can be any combination of a local file system
            # location and an S3 location. ie: fs_loc = '/local/path/' && s3_loc = None, etc.
            # EXCEPT: they can't both be None

            # TODO value is not None doesn't make sense, validate s3 url? Ignore and raise error later?
            if direction == 'Output' and value is not None:
                self.stageToS3 = True

            remote_work_path = os.environ.get('REMOTE_WORK_PATH', None)

            if remote_work_path is not None:
                self.__local_filesystem_location = None

                self.__remote_filesystem_location = os.path.join(
                    remote_work_path, direction.lower(), "%(port_name)s"
                )

                self.value = self.__remote_filesystem_location
            else:
                self.__remote_filesystem_location = None

                if value == '.':  # Shortcut for current working directory
                    self.__local_filesystem_location = os.getcwd()
                else:
                    self.__local_filesystem_location = value

                self.value = self.__local_filesystem_location

        else:
            self.value = value

    def __str__(self):
        if self.port_type == 'directory':
            return '%s: %s -- dir: %s' % (self.name, self.value, self.__local_filesystem_location)
        else:
            return '%s: %s' % (self.name, self.value)

    def __eq__(self, other):
        return self.name == other.name

    @property
    def port_type(self):
        return self.__task_json['type']

    @property
    def direction(self):
        return self.__direction

    @property
    def name(self):
        return self.__task_json['name']

    @name.setter
    def name(self, new_name):
        self.__task_json['name'] = new_name

        # Substitute in name of port for the remote path
        if self.value and '%(port_name)s' in self.value:
            self.value = self.value % {'port_name': new_name}

        # Make the dir
        if self.port_type == 'directory' and new_name != 'source_bundle':
            try:
                os.makedirs(self.value)
            except OSError:
                # if it exists, ignore 
                pass

    @property
    def remote_path(self):
        return self.__remote_filesystem_location

    @remote_path.setter
    def remote_path(self, new_path):
        self.__remote_filesystem_location = new_path

    @property
    def json(self):
        return self.__task_json

    @staticmethod
    def is_port(obj):
        """Predicate for inspect.getmodules function"""
        return isinstance(obj, Port)

    @property
    def path(self):
        if self.__local_filesystem_location is None and \
                self.__remote_filesystem_location is None and \
                self.port_type == 'directory':
            raise ValueError('Directory ports must be provided with a location')

        # If remote is True then use it no matter what
        if self.__remote_filesystem_location is not None:
            actual_path = self.__remote_filesystem_location % {'port_name': self.name}
            return actual_path
        else:
            return self.__local_filesystem_location

    def list_files(self, extensions=None):
        """
        List the ports contents by file type or all.
        :param extensions: string extensions, single string or list of extensions.
        :return: A list of full path names of each file.
        """
        if self.port_type.lower() != 'directory':
            raise ValueError("Port type is not == directory")

        filesystem_location = self.path

        for root, dirs, files in os.walk(filesystem_location):
            if extensions is None:
                return [os.path.join(root, f) for f in files]
            elif not isinstance(extensions, list):
                extensions = [extensions]

            subset_files = []

            for f in files:
                for extension in extensions:
                    if f.lower().endswith(extension.lower()):
                        subset_files.append(os.path.join(root, f))
                        break
            return subset_files

    def list_tiff(self):
        raise NotImplementedError
        # return self.list_files(extensions=['.TIFF', '.TIF', '.tiff', '.tif'])

    def list_1b(self):
        raise NotImplementedError
        # return self.list_files(extensions=['.TIL', '.til'])

    def write(self, filename, data, overwrite=False):

        if self.__direction != 'Output':
            raise ValueError('Only Output ports can be written to.')

        if self.port_type != 'directory':
            raise ValueError('Only Directory ports can be written to.')

        if os.path.isabs(filename) and os.path.isfile(filename):
            raise ValueError('File name must be relative to the Ports path')

        write_file = os.path.join(self.path, filename)

        file_mode = 'a' if not overwrite else 'w'

        self._check_or_create_dir(write_file)

        with open(write_file, file_mode) as out_file:
            out_file.write(data)

    @staticmethod
    def _check_or_create_dir(full_path_to_file):
        if os.path.isabs(full_path_to_file) and os.path.isfile(full_path_to_file):
            return
        else:
            folder = os.path.dirname(full_path_to_file)
            if not os.path.isdir(folder):
                os.makedirs(folder)

    @staticmethod
    def is_valid_filesys(path):
        if os.path.isabs(path) and os.path.isdir(path) and \
                not os.path.isfile(path):
            return True
        else:
            raise LocalPortValidationError(
                'Port value %s is not a valid filesystem location' % path
            )

    @staticmethod
    def is_valid_s3_url(url):
        """Checks if the url contains S3. Not an accurate validation of the url"""
        # Skip if the url start with source: (gbdxtools syntax)
        if url.startswith('source:'):
            return True

        scheme, netloc, path, _, _, _ = urlparse(url)

        port_except = RemotePortValidationError(
            'Port value %s is not a valid s3 location' % url
        )

        if len(scheme) < 2:
            raise port_except

        if 's3' in scheme or 's3' in netloc or 's3' in path:
            return True
        else:
            raise port_except


class InputPort(Port):
    """
    Helper Port subclass for Input Ports.
    """

    def __init__(self, value, port_type='directory', required=True):
        # TODO, need to allow source parameter for ports.
        super(InputPort, self).__init__(port_type, 'Input', required, value)


class OutputPort(Port):
    """
    Helper Port subclass for Output Ports.
    """

    def __init__(self, value=None, port_type='directory', required=True):
        super(OutputPort, self).__init__(port_type, 'Output', required, value)



import sys
import json
import uuid
import itertools
import time

from gbdx_cloud_harness.services.account_storage_service import AccountStorageService


class Workflow(object):
    """
    Class for building workflow objects that can generate a workflow definition
    from the cloud-harness object.
        Requirements:
            - build json workflow
            - execute workflow
            - succeeded property
            - complete property
            - monitor function
    """

    URL = "https://geobigdata.io/workflows/v1/workflows"
    STAGE_TO_S3 = {'inputs': [], 'taskType': 'StageDataToS3', 'name': 'StageToS3_%s' % str(uuid.uuid4())}

    def __init__(self, task, auth=None):

        if auth is None:
            # For testing
            self.storage = AccountStorageService()
            self.gbdx = self.storage.auth
        else:
            self.gbdx = auth
            self.storage = AccountStorageService(is_test=True)

        self.cloud_harness_task = task

        self.id = None
        self.status = None

    @property
    def json(self):
        return self._build_worklfow_json()

    def _build_worklfow_json(self):
        """
        Build a workflow definition from the cloud_harness task.
        """
        self.name = self.cloud_harness_task.name + '_' + str(uuid.uuid4())

        wf_json = {'tasks': [], 'name': 'cloud-harness_%s' % str(uuid.uuid4())}

        d = {
            "name": self.name,
            "outputs": [],
            "inputs": [],
            "taskType": self.cloud_harness_task.name
        }

        # Add input ports
        for port in self.cloud_harness_task.input_ports:
            port_value = port.value

            if port_value is False:
                port_value = 'false'
            if port_value is True:
                port_value = 'true'

            d['inputs'].append({
                "name": port.name,
                "value": port_value
            })

        # Add output ports
        for port in self.cloud_harness_task.output_ports:
            d['outputs'].append({
                "name": port.name
            })

        # Add task to workflow
        wf_json['tasks'].append(d)

        # Add port to be saved
        for port in self.cloud_harness_task.output_ports:
            # Add save data locations
            if port.stageToS3:
                save_location = '{customer_storage}/{run_name}/{port}'.format(
                    customer_storage=self.storage.location,
                    run_name=self.cloud_harness_task.run_name,
                    port=port.name
                )
                new_task = dict(**self.STAGE_TO_S3)
                new_task['inputs'] = [
                    {'name': 'data', 'source': '%s:%s' % (self.name, port.name)},
                    {'name': 'destination', 'value': save_location}
                ]
                wf_json['tasks'].append(new_task)

        return wf_json

    def execute(self, override_wf_json=None):
        """
        Execute the cloud_harness task.
        """
        r = self.gbdx.post(
            self.URL,
            json=self._build_worklfow_json() if override_wf_json is None else override_wf_json
        )

        try:
            r.raise_for_status()
        except:
            print("GBDX API Status Code: %s" % r.status_code)
            print("GBDX API Response: %s" % r.text)
            self.id = None
            return

        self.id = r.json()['id']
        self._refresh_status()

    @property
    def succeeded(self):
        self._refresh_status()
        return self.status['state'] == 'complete' and self.status['event'] == 'succeeded'

    @property
    def complete(self):
        self._refresh_status()
        return self.status['state'] == 'complete'

    def _refresh_status(self):
        r = self.gbdx.get('%s/%s' % (self.URL, self.id))
        self.status = r.json()['state']

    def monitor_run(self):
        """
        Monitor the workflows events and display spinner while running.
        :param workflow: the workflow object
        """

        spinner = itertools.cycle(['-', '/', '|', '\\'])

        while not self.complete:
            for i in xrange(300):
                sys.stdout.write(spinner.next())
                sys.stdout.flush()
                sys.stdout.write('\b')
                time.sleep(0.03)

        if self.succeeded:
            sys.stdout.write("\nWorkflow completed successfully\n")
            return True
        else:
            sys.stdout.write("\nWorkflow failed: %s\n" % self.status)
            return False



import json
from gbdx_auth import gbdx_auth


class TaskService(object):

    def __init__(self, auth=None):

        self.session = auth if auth is not None else gbdx_auth.get_session()

        self.task_url = 'https://geobigdata.io/workflows/v1/tasks'

    def register_task(self, task_def):
        '''
        Register a task for a python dict
        :param task_def: dict defining gbdx task
        '''
        r = self.session.post(
            self.task_url,
            data=task_def,
            headers={'Content-Type': 'application/json', 'Accept': 'application/json'}
        )

        task_dict = json.loads(task_def)

        if r.status_code == 200:
            return 'Task %s registered' % task_dict['name']
        else:
            return 'Task %s was not registered: %s' % (task_dict['name'], r.text)

    def delete_task(self, task_name):
        '''
        Delete a task from the platforms regoistry
        :param task_name: name of the task to delete
        '''
        response = self.session.delete('%s/%s' % (self.task_url, task_name))

        if response.status_code == 200:
            return 'Task %s deleted' % task_name
        elif response.status_code == 400:
            return None  # Task isn't registered.
        else:
            return 'Task %s was not deleted: %s' % (task_name, response.text)



import os

from gbdx_task_template.test.test_base import TestBase, temp_env_var
from gbdx_task_template import InputPort, OutputPort


class TestPort(TestBase):

    def test_port_list_files(self):
        path = os.path.dirname(os.path.realpath(__file__))

        port = InputPort(value=path)
        all_files = port.list_files()
        self.assertIn(__file__, all_files)
        for f in all_files:
            self.assertTrue(os.path.isfile(f))

        py_files = port.list_files(extensions=".py")

        self.assertIn(__file__, py_files)

        pyc_files = port.list_files(extensions=".pyc")

        self.assertNotIn(__file__, pyc_files)

        self.assertGreater(len(all_files), len(py_files))
        self.assertGreater(len(all_files), len(pyc_files))

    def test_port_list_files_bad_path(self):
        bad_path = 'not/good/path'
        port = InputPort(value=bad_path)
        try:
            port.list_files()
        except ValueError as e:
            self.assertIn('Port %s' % bad_path, e.message)

    def test_port_list_files_good_remote_path(self):
        good_path = os.path.dirname(os.path.realpath(__file__))
        bad_path = 'not/good/path'
        port = InputPort(value=bad_path)
        port.name = 'test'
        port.remote_path = good_path
        all_files = port.list_files(extensions=".py")
        self.assertEqual(len(all_files), 5)

    def test_port_write_good_path(self):
        """
        Test write to a port, relative to the port
        """
        good_path = os.path.dirname(os.path.realpath(__file__))
        port = OutputPort(value=good_path)

        # Write to file relative to port path.
        port.write('test.txt', 'Hello World')

        file_name = os.path.join(good_path, 'test.txt')

        with open(file_name, 'r') as f:
            self.assertEqual('Hello World', f.read())

    def test_port_write_good_path_abs(self):
        """
        Test write to a port, with an absolute filename
        """
        good_path = os.path.dirname(os.path.realpath(__file__))
        file_name = os.path.join(good_path, 'test.txt')

        port = OutputPort(value=good_path)

        # Write to abs path
        try:
            port.write(file_name, 'Hello World')
        except ValueError as e:
            self.assertIn('File name must be relative', e.message)

    def test_port_write_bad_port(self):
        good_path = os.path.dirname(os.path.realpath(__file__))
        port = InputPort(value=good_path)
        try:
            port.write('test.txt', 'Hello World')
        except ValueError as e:
            self.assertIn('Only Output ports', e.message)

    def test_port_write_bad_port2(self):
        good_path = os.path.dirname(os.path.realpath(__file__))
        port = OutputPort(port_type='string', value=good_path)
        try:
            port.write('test.txt', 'Hello World')
        except ValueError as e:
            self.assertIn('Only Directory ports', e.message)

    @temp_env_var(REMOTE_WORK_PATH='/mnt/work')
    def test_port_write_remote_path(self):
        good_path = os.path.dirname(os.path.realpath(__file__))
        port = OutputPort(value=good_path)
        port.name = 'test_port'
        self.assertEquals(port.path, '/mnt/work/output/test_port')

    def test_port_string(self):
        port = OutputPort(port_type='string')
        port.name = 'test_port'
        port.value = 10
        self.assertEquals(port.json['name'], 'test_port')
        self.assertEquals(port.json['type'], 'string')

    # def test_port_list_tiff(self):
    #     good_path = os.path.dirname(os.path.realpath(__file__))
    #     port = InputPort(value=good_path)
    #     tiff_files = port.list_tiff()
    #     self.assertEqual(len(tiff_files), 0)
    #
    #     all_files = []
    #
    #     file_name = os.path.join(good_path, 'test.tif')
    #     all_files.append(file_name)
    #     with open(file_name, 'w') as f:
    #         f.write('hi')
    #
    #     self.assertEqual(len(port.list_tiff()), 1)
    #
    #     file_name = os.path.join(good_path, 'test.TIFF')
    #     all_files.append(file_name)
    #     with open(file_name, 'w') as f:
    #         f.write('hi')
    #
    #     self.assertEqual(len(port.list_tiff()), 2)
    #
    #     file_name = os.path.join(good_path, 'test.TIL')
    #     all_files.append(file_name)
    #     with open(file_name, 'w') as f:
    #         f.write('hi')
    #
    #     self.assertEqual(len(port.list_tiff()), 2)
    #
    #     for name in all_files:
    #         os.remove(name)

    def tearDown(self):
        try:
            path = os.path.dirname(os.path.realpath(__file__))
            file_name = os.path.join(path, 'test.txt')
            os.remove(file_name)
        except Exception:
            pass



"""
cloud-harness Task Template
"""
import json
import os

from gbdx_task_template.gbdx_task_interface import GbdxTaskInterface


class TaskTemplate(GbdxTaskInterface):
    """ Base Class for custom tasks written for GBDX CLI.
    """
    task = None

    def __init__(self):

        self.__task_def_json = ''

        remote_work_path = os.environ.get('REMOTE_WORK_PATH', None)

        if remote_work_path is not None:
            self.__work_path = remote_work_path
            self.__remote_run = True
        else:
            self.__work_path = os.getcwd()
            self.__remote_run = False

        self.__src_path = None

        super(TaskTemplate, self).__init__(self.__work_path)

    def finalize(self, success_or_fail, message=''):
        """
        :param success_or_fail: string that is 'success' or 'fail'
        :param message:
        """
        if not self.__remote_run:
            return json.dumps({'status': success_or_fail, 'reason': message}, indent=4)
        else:
            super(TaskTemplate, self).finalize(success_or_fail, message)

    def __enter__(self):

        if self.task is None:
            raise ValueError('A task must be initialized before running a TaskTemplate subclass.')

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            # print(self.finalize('failed', str(exc_val)))# Commented out for gbdxtools demo.
            self.finalize('failed', str(exc_val))
        else:
            # Write output ports to ports.json
            input_ports, output_ports = self.task.ports
            for port in output_ports:
                if port.port_type == 'string':
                    self.set_output_string_port(port.name, port.value)

            # print(self.finalize('success', self._reason)) # Commented out for gbdxtools demo.
            self.finalize('success', self._reason)



#!/usr/bin/python
"""
Command Line interface for running remote workflows from client workstations

Usage:
    cloud-harness create <dir_name> [--destination=<path>]
    cloud-harness run <file_name> [--remote] [--verbose] [--upload] [--download] [--dry-run]

Options:
    --verbose           List all the details pertaining.
    --remote            Run the application on the configured environment
    --destination=<path>       Override the location where the application will be created
    --upload            Uploads all the local task ports to S3.
    --download          Downloads the output ports to the local filesystem.
    --dry-run           Do not execute the task.

"""
"""
Notes:

- Default behaviour:
    1) If app is ran LOCALLY,
        then run on local data or raise file not found error.
    2) If app is ran REMOTELY and upload flag is provided,
        push ports data to S3, substitute in workflow value.
    3) If app is ran REMOTELY and NO upload flag,
        Only push source bundle, substitute in workflow. Assumed other port
        values are valid S3 locations. Validate

"""
from docopt import docopt
import imp
import inspect
import os
import warnings
import tarfile
import shutil
import json
import uuid
from datetime import datetime

from gbdx_task_template import TaskTemplate
from gbdx_cloud_harness.services.port_service import PortService
from gbdx_cloud_harness.services.task_service import TaskService
from gbdx_cloud_harness.utils.printer import printer
from gbdx_cloud_harness.workflow import Workflow

__author__ = 'michaelconnor'


class TaskController(object):

    RESOURCE_NAME = 'app'

    TEMPLATE_FOLDER = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates')

    TEMPLATE_FILENAME = 'task_template.py'

    SOURCE_BUNDLE_PORT = 'source_bundle'

    DEFAULT_NEW_APP_FILENAME = 'app.py'

    IGNORE_FILES_NAME = 'pkg_ignore'

    def __init__(self, arguments):

        self._arguments = arguments

        self.FUNCTION_KEYS = {
            'create': self._create_app,
            'run': self._run_app,
        }

    def invoke(self):
        """
        Execute the command from the arguments.
        :return: None or Error
        """
        for key in self.FUNCTION_KEYS.keys():
            if self._arguments[key] is True:
                self.FUNCTION_KEYS[key]()

    def _create_app(self):
        """
        Method for creating a new Application Template.
        USAGE: cloud-harness create <dir_name> [--destination=<path>]
        """
        template_path = os.path.join(
            os.path.dirname(os.path.dirname(os.path.abspath(__file__))), self.TEMPLATE_FOLDER, self.TEMPLATE_FILENAME
        )

        new_dir = self._arguments['<dir_name>']

        # Make new application directory
        override_destination = self._arguments.get('--destination', None)
        if override_destination is not None:
            if override_destination == '':
                raise ValueError('Destination path is empty')
            # Check if the new destination is abs and exists.
            if os.path.isabs(override_destination) and os.path.isdir(override_destination):
                new_dir = os.path.join(override_destination, new_dir)
            else:
                # Create a path from the cwd, then check if it is valid and exists.
                override_path = os.path.join(os.getcwd(), override_destination)
                if not os.path.isabs(override_path) or not os.path.isdir(override_path):
                    raise ValueError('New path parameter %s is not a directory' % override_destination)
                new_dir = os.path.join(override_path, new_dir)

        else:
            if os.path.isabs(new_dir) or os.path.sep in new_dir:
                raise ValueError("Directory name is invalid")
            # No override, put the folder in the cwd.
            new_dir = os.path.join(os.getcwd(), new_dir)

        os.makedirs(new_dir)

        new_file_path = os.path.join(new_dir, self.DEFAULT_NEW_APP_FILENAME)

        # Copy the template the new application location.
        shutil.copyfile(template_path, new_file_path)

        printer('New Application created at %s' % new_file_path)

    def _run_app(self):
        """
        Method for running a custom Application Templates.
        NOTES:
            * The default name of the application is app.py. So this function is going to look
            for app.py, unless the --file option is provide with a different file name.
            * The generated source bundle will package everything in the work_path. If large files
            not required for the application source, they need to be ignored. Use a file called "pkg_ignore"
            to identify folders and files to ignore.
        USAGE: cloud-harness run <file_name> [--remote] [--verbose] [--upload] [--download] [--dry-run]
        """
        is_remote_run = self._arguments.get('--remote')
        filename = self._arguments.get('<file_name>')
        upload_ports = self._arguments.get('--upload')
        download_ports = self._arguments.get('--download')
        is_verbose = self._arguments.get('--verbose')
        # A dry run means, allow port sot be pushed up, but don't allow execution and monitoring.
        is_dry_run = self._arguments.get('--dry-run')

        if download_ports:  # TODO temporary until implemented.
            raise NotImplementedError("Downloading of output ports is not implemented yet.")

        # Check if the filename passed is actually a class object (gbdxtools functionality)
        if not isinstance(filename, str) and issubclass(filename, TaskTemplate):
            template_class = filename
            template_file = inspect.getfile(template_class)
            config_file = self._write_config_file(template_file)

        else:
            template_file = self._get_template_abs_path(filename)

            if not os.path.isfile(template_file):
                raise ValueError('The location %s does not exist' % template_file)

            config_file = self._write_config_file(template_file)

            template_class = self._get_class(template_file)

        with template_class() as template:
            if is_remote_run:
                task = template.task

                # Set the source bundle directory to where the tempalte_file is.
                task.source_bundle.value = os.path.join(os.path.dirname(template_file), 'tmp_%s' % str(uuid.uuid4()))

                # Create a task service object
                task_service = TaskService()
                task_service.delete_task(task.name)
                printer(task_service.register_task(task.json()))

                task.run_name = '{timestamp}_{task_name}'.format(
                    task_name=task.name,
                    timestamp=datetime.utcnow().strftime('%Y_%m_%d_%H')
                )

                src_bundle_dir = task.source_bundle.value

                # Create source bundle to be executed on the GBDX platform
                self._archive_source(os.path.dirname(src_bundle_dir), src_bundle_dir)

                port_service = PortService(task)

                if upload_ports:
                    # Push all port data to S3
                    port_service.upload_input_ports()
                else:
                    # Only push source bundle port
                    port_service.upload_input_ports(port_list=[self.SOURCE_BUNDLE_PORT])

                # Delete source bundle directory and config after upload.
                shutil.rmtree(src_bundle_dir)
                os.remove(config_file)

                # Build task json to run remotely
                self.task = port_service.task

                # Validate task
                task.is_valid(remote=True)

                workflow = Workflow(self.task)

                if is_verbose:
                    printer(template.task.json())

                    temp_wf = workflow.json

                    printer(temp_wf)

                if not is_dry_run:
                    try:
                        workflow.execute()
                        printer(workflow.id)
                    except Exception as e:
                        printer(e.message)
                        template.reason = "Execution Failed: %s" % e.message
                        return

                    # Monitor events of workflow
                    is_done = workflow.monitor_run()

                    if is_done:
                        template.reason = "Execution Completed"
                    else:
                        template.reason = "Execution Failed during Run"

                    if download_ports:
                        # port_service.download_output_port()
                        pass

                    # Note: This may be temporary while working with gbdxtools
                    # Delete task after run
                    task_service.delete_task(task.name)

            else:
                # Validate task
                template.task.is_valid()

                if is_verbose:
                    printer(template.task.json())
                    all_ports = template.task.ports[0] + template.task.ports[1]
                    printer([port.__str__() for port in all_ports])

                if not is_dry_run:
                    # Run Task Locally
                    try:
                        template.invoke()
                    except Exception as e:
                        template.reason = "Failed Exception: %s" % e

                    template.reason = "Execution Completed"
                else:
                    template.reason = "Execution Skipped"

    @staticmethod
    def _write_config_file(template_file):
        """
        Write a config file to the source bundle location to identify the entry point.
        :param template_file: path to the task template subclass (executable)
        """
        config_filename = '.cloud_harness_config.json'
        config_path = os.path.dirname(template_file)
        curr_path = os.getcwd()

        # filename = os.path.split(template_file)[1]
        filename = template_file[len(curr_path) + 1:]

        config_file = os.path.join(config_path, config_filename)

        with open(config_file, 'w') as f:
            f.write(json.dumps({'task_filename': filename}))

        return config_file

    @staticmethod
    def _get_class(template_file):
        """
        Import the file and inspect for subclass of TaskTemplate.
        :param template_file: filename to import.
        """
        with warnings.catch_warnings():
            # suppress warning from importing
            warnings.filterwarnings("ignore", category=RuntimeWarning)
            template_module = imp.load_source('module.name', template_file)

        # Find the subclass of TaskTemplate
        for name, data in inspect.getmembers(template_module, inspect.isclass):
            if issubclass(data, TaskTemplate) and data.__name__ != TaskTemplate.__name__:
                return data

    @staticmethod
    def _get_template_abs_path(filename):
        """
        Return a valid absolute path. filename can be relative or absolute.
        """
        if os.path.isabs(filename) and os.path.isfile(filename):
            return filename
        else:
            return os.path.join(os.getcwd(), filename)

    @staticmethod
    def _archive_source(source_folder, destination):

        write_mode = 'w:gz'

        global ignore_location
        ignore_location = source_folder

        if not os.path.isabs(destination) and not os.path.isdir(destination):
            raise ValueError("Invalid destination folder %s" % destination)

        new_dest_filename = os.path.join(destination, 'archive.tar.gz')

        try:
            os.makedirs(destination)
        except OSError as e:
            if 'File exists' in e.message:
                shutil.rmtree(destination)

        with tarfile.open(new_dest_filename, write_mode) as src_archive:
            src_archive.add(source_folder, arcname="", filter=TaskController._filter_archive)

    @staticmethod
    def _filter_archive(tarinfo):

        ignore_filename = os.path.join(ignore_location, TaskController.IGNORE_FILES_NAME)

        if TaskController.SOURCE_BUNDLE_PORT in tarinfo.name:
            return None

        # Ignore pyc files.
        if tarinfo.name.endswith('.pyc'):
            return None

        if not os.path.isfile(ignore_filename):
            return tarinfo

        with open(ignore_filename, 'rb') as f:
            for ignore in f.readlines():
                # Only check if the ignore is a folder (startswith) or an extension (endswith)
                ignore = ignore.rstrip().rstrip(os.path.sep)
                if tarinfo.name.startswith(ignore) or tarinfo.name.endswith(ignore):
                    return None

            return tarinfo


def main():
    arguments = docopt(__doc__)
    app = TaskController(arguments)
    app.invoke()


if __name__ == '__main__':
    main()



import os

from gbdx_cloud_harness.services.account_storage_service import AccountStorageService
from gbdx_cloud_harness.utils.printer import printer


class PortService(object):

    def __init__(self, task, storage_service=None):

        self._task = task

        if storage_service is None:
            self.storage = AccountStorageService()
        else:
            # Override for the storage service
            self.storage = storage_service

        self.s3_root = self.storage.location

    @property
    def task(self):
        return self._task

    def upload_input_ports(self, port_list=None, exclude_list=None):
        """
        Takes the workflow value for each port and does the following:
            * If local filesystem -> Uploads locally files to s3.
                S3 location will be as follows:
                    gbd-customer-data/<acct_id>/<workflow_name>/<task_name>/<port_name>/
            * If S3 url -> do nothing.
        :returns the update workflow with S3 urls.
        """

        input_ports = self._task.input_ports

        for port in input_ports:

            # If port list is not None, then only allow port names in the list
            if port_list and port.name not in port_list:
                continue

            # Exclude ports as provided
            if exclude_list and port.name in exclude_list:
                continue

            # port_value = port.get('value', None)

            # Check if the port value is a valid file system location
            if not port.value or not os.path.isabs(port.value) or not os.path.isdir(port.value):
                continue

            # The prefix for each key that is uploaded, not including the the acct id.
            prefix = '{run_name}/{port}'.format(
                run_name=self._task.run_name,
                # task=self._task.name,
                port=port.name
            )

            port_files = self._get_port_files(port.value, prefix)

            # Update the port value with an S3 url
            port.value = '%s/%s' % (self.s3_root, prefix)

            if len(port_files) == 0:
                printer('Port %s is empty, push to S3 skipped' % port.name)
            else:
                self.storage.upload(port_files)
                printer('Port %s pushed to account storage, %s files' % (port.name, len(port_files)))

    @staticmethod
    def _get_port_files(local_path, prefix):
        """
        Find files for the local_path and return tuples of filename and keynames
        :param local_path: the local path to search for files
        :param prefix: the S3 prefix for each key name on S3
        """
        source_files = []

        for root, dirs, files in os.walk(local_path, topdown=False):

            for name in files:
                fname = os.path.join(root, name)

                key_name = '%s/%s' % (prefix, fname[len(local_path) + 1:])

                source_files.append((fname, key_name))

        return source_files

    # TODO Implement
    # def download_output_port(self, workflow, arg_outputs=[]):
    #     """
    #     Download the outputs for the given workflow from s3, maintaining
    #     the namespace structure.
    #     :param workflow:
    #     :param arg_outputs:
    #     """
    #     output_location = os.path.join(os.getcwd(), 'Workflows', workflow['name'], 'outputs')
    #
    #     # Get input directories
    #     outputs = next(os.walk(output_location))[1]
    #
    #     # Filter input dirs by workflow inputs
    #     if len(arg_outputs) > 0:
    #         outputs = set(outputs).intersection(arg_outputs)
    #
    #     for next_output in outputs:
    #         s3_folder = '%s/%s' % (workflow['name'], next_output)
    #         dir_path = os.path.join(output_location, next_output)
    #         # Get all keys contained in the folder
    #         try:
    #             key_names = self.storage.list(s3_folder=s3_folder)
    #         except ClientError:
    #             # TODO is this the only reason this will fail?
    #             raise ValueError("Account Storage location does not exist")
    #
    #         self.storage.download(dir_path, key_names)



"""
Copied from gbdxtools.
This function returns a mock gbdx-auth requests session with a dummy token.  You can optionally pass in a real token
if you want to actually make requests.
"""

from future import standard_library
standard_library.install_aliases()

from oauthlib.oauth2 import LegacyApplicationClient
from requests_oauthlib import OAuth2Session


def get_mock_gbdx_session(token='dummytoken'):
    s = OAuth2Session(
        client=LegacyApplicationClient('asdf'),
        auto_refresh_url='fdsa',
        auto_refresh_kwargs={
            'client_id':'asdf',
            'client_secret':'fdsa'
        }
    )

    s.token = token
    s.access_token = token
    return s



import json
import sys


def printer(data):
    """
    response is json or straight text.
    :param data:
    :return:
    """
    data = str(data) # Get rid of unicode
    if not isinstance(data, str):
        output = json.dumps(
            data,
            sort_keys=True,
            indent=4,
            separators=(',', ': ')
        )
    elif hasattr(data, 'json'):
        output = data.json()
    else:
        output = data

    sys.stdout.write(output)
    sys.stdout.write('\n')
    sys.stdout.flush()
    return



import os
import tarfile
import shutil


from gbdx_task_template.test.test_base import TestBase
from gbdx_cloud_harness import TaskController


class TestApp(TestBase):

    def setUp(self):
        self.DIRS_TO_REMOVE = []
        self.FILES_TO_REMOVE = []

    def test_get_filename_pos(self):
        app = TaskController(None)

        # filename only.
        test_path1 = app._get_template_abs_path('app.py')
        exp_result1 = os.path.join(os.getcwd(), 'app.py')
        self.assertEqual(exp_result1, test_path1)

        # Test filename is absolute
        test_filename2 = os.path.abspath(__file__)
        test_path2 = app._get_template_abs_path(test_filename2)
        self.assertEqual(test_filename2, test_path2)

    def test_archive_pos_filter(self):
        test_path_dest = os.path.dirname(os.path.realpath(__file__))
        test_path_src = os.path.join(test_path_dest, 'input')

        filter_file = os.path.join(test_path_src, TaskController.IGNORE_FILES_NAME)
        self.FILES_TO_REMOVE.append(filter_file)
        with open(filter_file, 'a') as f:
            f.write('imgs/\n')
            f.write('README.md\n')

        TaskController._archive_source(test_path_src, test_path_dest)

        arch_tar = tarfile.open(os.path.join(test_path_dest, 'archive.tar.gz'))
        self.FILES_TO_REMOVE.append(arch_tar.name)
        filenames = arch_tar.getnames()
        self.assertIn('app.py', filenames)
        self.assertNotIn('README.md', filenames)
        self.assertIn('data/readme.txt', filenames)
        self.assertIn('data/subsub/test.rst', filenames)
        self.assertNotIn('imgs/tester.png', filenames)

    def test_archive_pos(self):

        test_path_dest = os.path.dirname(os.path.realpath(__file__))
        test_path_src = os.path.join(test_path_dest, 'input')

        TaskController._archive_source(test_path_src, test_path_dest)

        arch_tar = tarfile.open(os.path.join(test_path_dest, 'archive.tar.gz'))
        self.FILES_TO_REMOVE.append(arch_tar.name)
        filenames = arch_tar.getnames()
        self.assertIn('app.py', filenames)
        self.assertIn('README.md', filenames)
        self.assertIn('data/readme.txt', filenames)
        self.assertIn('data/subsub/test.rst', filenames)
        self.assertIn('imgs/tester.png', filenames)

    def test_create_neg(self):

        bad_dir_names = [
            '/new_test_app',  # abs paths not allowed
            'my/dir',  # No path seperators allowed.
        ]

        args = {
            '--destination': None,
            '--download': False,
            '--remote': False,
            '--upload': False,
            '--verbose': False,
            '<file_name>': None,
            'create': True,
            'run': False
        }

        for baddir in bad_dir_names:
            args['<dir_name>'] = baddir
            app_ctrl = TaskController(args)

            try:
                app_ctrl.invoke()
                self.assertTrue(False)
            except ValueError as e:
                self.assertTrue('Directory name is invalid' in e.message)

    def test_create_pos(self):

        curr_path = os.getcwd()
        dir_name = 'new_test_dir'
        dir_path = os.path.join(curr_path, dir_name)
        self.DIRS_TO_REMOVE.append(dir_path)

        args = {
            '--destination': None,
            '--download': False,
            '--remote': False,
            '--upload': False,
            '--verbose': False,
            '<file_name>': None,
            '<dir_name>': dir_name,
            'create': True,
            'run': False
        }

        app_ctrl = TaskController(args)
        app_ctrl.invoke()

        exp_result = os.path.join(dir_path, TaskController.DEFAULT_NEW_APP_FILENAME)

        self.assertTrue(os.path.isdir(dir_path))
        self.assertTrue(os.path.isfile(exp_result))

    def test_create_dest_neg(self):

        dir_name = 'new_test_dir'

        bad_destinations = [
            'relative/path',  # relative paths have to exist
            ''  # Empty string not allowed
        ]

        args = {
            '--download': False,
            '--remote': False,
            '--upload': False,
            '--verbose': False,
            '<file_name>': None,
            '<dir_name>': dir_name,
            'create': True,
            'run': False
        }

        for baddest in bad_destinations:
            args['--destination'] = baddest
            app_ctrl = TaskController(args)

            try:
                app_ctrl.invoke()
                self.assertTrue(False)
            except ValueError as e:
                self.assertTrue('not a directory' in e.message or 'path is empty' in e.message)

    def test_create_dest_pos(self):
        # Test with relative destination.
        curr_path = os.path.dirname(os.path.abspath(__file__))
        dir_name = 'new_test_dir'
        dest_name = curr_path[len(os.getcwd()) + 1:]
        dir_path = os.path.join(curr_path, dir_name)
        self.DIRS_TO_REMOVE.append(dir_path)

        args = {
            '--destination': dest_name,
            '--download': False,
            '--remote': False,
            '--upload': False,
            '--verbose': False,
            '<file_name>': None,
            '<dir_name>': dir_name,
            'create': True,
            'run': False
        }

        app_ctrl = TaskController(args)
        app_ctrl.invoke()

        exp_result = os.path.join(dir_path, TaskController.DEFAULT_NEW_APP_FILENAME)

        self.assertTrue(os.path.isdir(dir_path))
        self.assertTrue(os.path.isfile(exp_result))

    def test_create_dest_pos_abs(self):
        # Test with absolute destination.
        dir_name = 'new_test_dir'
        dest_name = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'input', 'data')
        dir_path = os.path.join(dest_name, dir_name)
        self.DIRS_TO_REMOVE.append(dir_path)

        args = {
            '--destination': dest_name,
            '--download': False,
            '--remote': False,
            '--upload': False,
            '--verbose': False,
            '<file_name>': None,
            '<dir_name>': dir_name,
            'create': True,
            'run': False
        }

        app_ctrl = TaskController(args)
        app_ctrl.invoke()

        exp_result = os.path.join(dir_path, TaskController.DEFAULT_NEW_APP_FILENAME)

        self.assertTrue(os.path.isdir(dir_path))
        self.assertTrue(os.path.isfile(exp_result))

    def test_run_pos(self):

        filename = os.path.join('examples', 'echo_task', 'app.py')

        args = {
            '--destination': None,
            '--download': False,
            '--remote': False,
            '--upload': False,
            '--verbose': True,
            '<file_name>': filename,
            '<dir_name>': None,
            'create': False,
            'run': True
        }

        app_ctrl = TaskController(args)
        app_ctrl.invoke()
        self.assertTrue(True)  # Test passes if no exception raised...

        # test with filename the doesn't exist
        args['<file_name>'] = 'does_not_exist.py'

        app_ctrl = TaskController(args)

        try:
            app_ctrl.invoke()
        except ValueError as e:
            self.assertTrue('does not exist' in e.message)

    def tearDown(self):

        for filename in self.FILES_TO_REMOVE:
            self._remove_file(filename)

        for dirname in self.DIRS_TO_REMOVE:
            shutil.rmtree(dirname)

    @staticmethod
    def _remove_file(filename):

        if not os.path.isfile(filename):
            return

        os.remove(filename)



import json

from gbdx_task_template.test.test_base import TestBase, MyBasicApp, BadTaskApp


class TestApplicationTemplate(TestBase):

    def test_app(self):
        with MyBasicApp() as task:
            task.invoke()
            task.reason = "All ok"

            self.assertTrue(self._is_json_valid(task.task.json()))

    def test_bad_app(self):
        try:
            with BadTaskApp() as task:
                task.invoke()
        except Exception as e:
            self.assertIn('MyCustomTask', e.message)

    @staticmethod
    def _is_json_valid(json_str):
        try:
            json.loads(json_str)
            return True
        except:
            return False



import os

from gbdx_cloud_harness.test.conftest import my_vcr, TestApp
from gbdx_cloud_harness.test.auth_mock import get_mock_gbdx_session
from gbdx_cloud_harness.services.task_service import TaskService

test_app = TestApp


@my_vcr.use_cassette('gbdx_cloud_harness/services/test/vcr_cassettes/task_service.yaml', filter_headers=['authorization'])
def test_task_register_delete(test_app):

    cassette_exists = os.path.isfile(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'vcr_cassettes', 'task_service.yaml')
    )

    with test_app() as app:

        if cassette_exists:
            # Use mock session, so CI doesn't fail (creds)
            test_auth = get_mock_gbdx_session()
            task_service = TaskService(auth=test_auth)
        else:
            # Create real session
            task_service = TaskService()

        message = task_service.register_task(app.task.json())
        print(message)
        assert 'not registered' not in message

        message = task_service.delete_task(app.task.name)
        print(message)
        assert 'not deleted' not in message



import os
import json
from unittest import TestCase
from moto import mock_s3

from gbdx_cloud_harness.services.port_service import PortService
from gbdx_cloud_harness.services.account_storage_service import AccountStorageService
from gbdx_task_template.test.test_base import MyBasicApp, MY_BASIC_APP_WF_DEF


class TestPortService(TestCase):

    def setUp(self):
        self.mock = mock_s3()
        self.mock.start()

        self.test_bucket = 'not_provided'
        self.test_account_id = 'not_provided'

        self.test_storage = AccountStorageService(is_test=True)

        self.test_wf = {
            "name": "test",
            "tasks": []
        }

        self.test_path = os.path.dirname(os.path.realpath(__file__))
        self.source_bundle = os.path.join(
            os.getcwd(),
            'source_bundle'
        )
        try:
            os.makedirs(self.source_bundle)
        except OSError:
            pass

        print('Directory %s has been created' % self.source_bundle)

    def tearDown(self):
        # self.mock.stop()

        os.rmdir(self.source_bundle)

    def test_port_file_search(self):

        test_prefix = 'My/Test/Prefix'
        source_files = PortService._get_port_files(
            self.test_path,
            test_prefix
        )

        exp_result = (
            os.path.join(self.test_path, 'input', 'data', 'readme.txt'),
            '%s/%s' % (test_prefix, 'input/data/readme.txt')
        )
        self.assertIn(exp_result, source_files)

    def test_upload_port(self):
        """
        Test single task and port
        """

        new_path = os.path.join(self.test_path, 'input')

        test_task_def = MY_BASIC_APP_WF_DEF['tasks'][0]

        task_run_name = 'upload_test_run'

        # prefix is {task_name}/{port_name}
        test_prefix = '/'.join(
            ['not_provided', task_run_name,
             test_task_def['inputs'][0]['name']]
        )

        exp_results = [
            'data/readme.txt',
            'data/subsub/test.rst',
            'imgs/tester.png',
            # '.DS_Store',
            'README.md',
            'app.py',
            # '__pycache__/app.cpython-27-PYTEST.pyc'
        ]

        with MyBasicApp() as app:

            app.task.second_port.value = new_path
            app.task.run_name = task_run_name

            port_service = PortService(
                app.task,
                storage_service=self.test_storage
            )

            port_service.upload_input_ports()

            new_task = port_service.task

            self.assertTrue(new_task.is_valid(remote=True))

            bucket_contents = self.test_storage.client.list_objects(Bucket=self.test_bucket)['Contents']
            print([key['Key'] for key in bucket_contents])
            count = 0
            for key in bucket_contents:
                self.assertEqual(len(bucket_contents), len(exp_results))
                for result in exp_results:
                    if key['Key'].endswith(result):
                        self.assertTrue(True)
                        count += 1

                self.assertIn(self.test_account_id, key['Key'])
                self.assertIn(test_prefix, key['Key'])

            self.assertEqual(count, len(bucket_contents))

            self.assertTrue(new_task.second_port.value.startswith(self.test_storage.location))
            self.assertTrue(new_task.second_port.value.endswith(test_prefix))

    def test_upload_ports(self):
        """
        Test multiple tasks and ports
        """

        new_path = os.path.join(self.test_path, 'input')

        with MyBasicApp() as app:

            app.task.second_port.value = new_path
            app.task.run_name = 'uploadss_test_run'

            port_service = PortService(
                app.task,
                storage_service=self.test_storage
            )


            port_service.upload_input_ports()

            new_task = port_service.task

            self.assertTrue(new_task.is_valid(remote=True))

            bucket_contents = self.test_storage.client.list_objects(Bucket=self.test_bucket)['Contents']

            count_files = [files for root, dirs, files in os.walk(new_path)]

            self.assertEqual(len(bucket_contents), sum([len(x) for x in count_files]))

            cnt_port1 = 0
            cnt_port2 = 0
            cnt_port3 = 0

            for key in bucket_contents:
                if 'port4' in key['Key']:
                    raise ValueError("Key name is incorrect %s" % key['Key'])
                if 'port1' in key['Key']:
                    cnt_port1 += 1
                if 'port2' in key['Key']:
                    cnt_port2 += 1
                if 'port3' in key['Key']:
                    cnt_port3 += 1

            self.assertTrue(cnt_port1 == cnt_port2 == cnt_port3)

    # def test_update_output_ports(self):
    #
    #     self.test_wf['tasks'].append(
    #         {
    #             "name": "task1",
    #             "inputs": [
    #                 {"name": "test_port", "value": os.path.join(self.test_path, 'input')}
    #             ],
    #             "outputs": [
    #                 {"name": "out1", "value": '{_s3_output_folder}/out1'},
    #                 {"name": "out2", "value": '{_s3_output_folder}/out2'},
    #                 {"name": "out3", "value": 'Not a directory port'}
    #             ]
    #         }
    #     )
    #
    #     port_service = PortService(
    #         json.dumps(self.test_wf),
    #         storage_service=self.test_storage
    #     )
    #
    #     test_prefix = '/'.join(
    #         [self.test_wf['name'], self.test_wf['tasks'][0]['name']]
    #     )
    #     print(test_prefix)
    #
    #     port_service.update_output_port_values()
    #
    #     new_workflow = port_service.workflow
    #
    #     output_ports = new_workflow['tasks'][0]['outputs']
    #
    #     s3_prefix = 's3://'
    #     for port in output_ports:
    #         if port['value'].startswith(s3_prefix):
    #             postfix = '/'.join([test_prefix, port['name']])
    #             self.assertTrue(port['value'].endswith(postfix))



import os
import json


class GbdxTaskInterface(object):
    """A GBDX task base class.
    """

    def __init__(self, work_path="/mnt/work/"):
        self.__work_path = work_path
        self.__string_input_ports = None
        self.__string_output_ports = None
        self._reason = None

        if not os.path.exists(self.__work_path):
            raise Exception("Working path must exist. {_path}.".format(_path=self.__work_path))

        string_input_ports = os.path.join(self.__work_path, 'input', "ports.json")
        if os.path.exists(string_input_ports):
            with open(string_input_ports, 'r') as f:
                self.__string_input_ports = json.load(f)

    @property
    def base_path(self):
        return self.__work_path

    @property
    def input_path(self):
        return os.path.join(self.base_path, 'input')

    @property
    def output_path(self):
        return os.path.join(self.base_path, 'output')

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, reason):
        self._reason = reason

    def get_input_string_port(self, port_name, default=None):
        """
        Get input string port value
        :param port_name:
        :param default:
        :return: :rtype:
        """
        if self.__string_input_ports:
            return self.__string_input_ports.get(port_name, default)
        return default

    def get_input_data_port(self, port_name):
        """
        Get the input location for a specific port
        :param port_name:
        :return: :rtype:
        """
        return os.path.join(self.input_path, port_name)

    def get_output_data_port(self, port_name):
        """
        Get the output location for a specific port
        :param port_name:
        :return: :rtype:
        """
        return os.path.join(self.output_path, port_name)

    def set_output_string_port(self, port_name, value):
        """
        Set output string port value
        :param port_name:
        :param value:
        :return: :rtype:
        """
        if not self.__string_output_ports:
            self.__string_output_ports = {}

        self.__string_output_ports[port_name] = value

    def invoke(self):
        """
        The do something method
        :rtype : bool
        :raise RuntimeError:
        """
        raise RuntimeError("JobRunner Baseclass invoke is not callable")

    def finalize(self, success_or_fail, message=''):
        """
        :param success_or_fail: string that is 'success' or 'fail'
        :param message:
        """
        if self.__string_output_ports:
            with open(os.path.join(self.output_path, 'ports.json'), 'w') as opf:
                json.dump(self.__string_output_ports, opf, indent=4)

        with open(os.path.join(self.base_path, 'status.json'), 'w') as sf:
            json.dump({'status': success_or_fail, 'reason': message}, sf, indent=4)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            self.finalize('failed', str(exc_val))
        else:
            self.finalize('success', self._reason)



import os
import json


class TaskDescriptor():
    with open(os.path.abspath(
            os.path.join(os.path.dirname(os.path.realpath(__file__)),
            "task_schema.json")
    )) as json_file:
        td_schema = json.load(json_file)

    @staticmethod
    def get_schema():
        return TaskDescriptor.td_schema



"""
This file is an example of an application that can be run with the GBDX CLI both locally and remotely.

To run the application: gbdx app run

"""
from gbdx_task_template import TaskTemplate, Task, InputPort, OutputPort


class ExampleApp(TaskTemplate):
    """
    The class for the example application. All GBDX Cloud Harness applications must be a subclass
     of TaskTemplate.
    """

    # Create a new Task for the application, takes a string which is the tasks name.
    task = Task("MyTask")

    # Add an input port of type directory to the
    task.input_data = InputPort(port_type="directory", value="/my/local/input/data")

    # Add an output port of type directory to the task.
    task.output_data = OutputPort(
        port_type="directory", value="/my/local/output/location"
    )

    # OPTIONALLY, output_data can be initialized with an S3 location for remote runs only.
    # task.output_data = OutputPort(
    #     port_type="directory", value="s3://mybucket/folder"
    # )

    # Add an output port of type string.
    task.output_param = OutputPort(
        port_type="string"
    )

    def invoke(self):

        # List files of a port with an extension filter.
        my_files = self.task.output_data.list_files(extensions=['.py', '.tif'])

        # Write Task Magic Here...

        msg = 'Found %s files in %s' % (len(my_files), self.task.input_data.value)
        print(msg)  # Send num files to stdout

        # Save value to output parameter
        self.task.output_param.value = len(my_files)

        # Write new files to an Output port that is type directory.
        self.task.output_data.write('my_output.txt', msg, overwrite=True)

        self.reason = 'Everything Completed Correctly'



import os
import json

from jsonschema import validate as validate_schema

from gbdx_task_template.schemas.task_descriptor import TaskDescriptor
from gbdx_task_template.port import Port, InputPort, OutputPort


class Task(object):
    """
    Class to encapsulate the Task configuration
    """

    def __init__(self, name, docker_image=None):
        self.__task_json = {
            "containerDescriptors": [{
                "type": "DOCKER",
                "properties": {"image": "tdgp/sdk_application_runner:latest"}
            }],
            "description": "GBDX SDK Custom Task",
            "inputPortDescriptors": [],
            "outputPortDescriptors": [],
            "properties": {"timeout": 7200},
            "name": name
        }

        if docker_image is not None:
            self.__task_json['containerDescriptors'][0]['properties']['image'] = docker_image

        # Add default source bundle port
        port_name = "source_bundle"
        dest_path = os.path.join(os.getcwd(), port_name)
        srcbundle = InputPort(value=dest_path)
        srcbundle.name = port_name
        self.source_bundle = srcbundle

    def __setattr__(self, key, value):
        # Check if key already exists
        if Port.is_port(value):
            value.name = key
            self.add_port(value)

        super(Task, self).__setattr__(key, value)

    @property
    def src_path(self):
        return self.source_bundle.value

    @property
    def name(self):
        return self.__task_json['name']

    @name.setter
    def name(self, value):
        self.__task_json['name'] = value

    @property
    def properties(self):
        return self.__task_json['properties']

    @properties.setter
    def properties(self, value):
        self.__task_json['properties'].update(value)

    @property
    def input_ports(self):
        return self.__task_json['inputPortDescriptors']

    @property
    def output_ports(self):
        return self.__task_json['outputPortDescriptors']

    @property
    def ports(self):
        return self.__task_json['inputPortDescriptors'], self.__task_json['outputPortDescriptors']

    def json(self):
        return json.dumps(self.__task_json, cls=CustomEncoder, indent=2)

    def add_port(self, new_port):
        if not Port.is_port(new_port):
            raise TypeError("Task ports must be of type Port")

        if new_port.direction == 'Input':
            self.__task_json['inputPortDescriptors'].append(new_port)
        elif new_port.direction == 'Output':
            self.__task_json['outputPortDescriptors'].append(new_port)

    def is_valid(self, remote=False):
        schema = TaskDescriptor().get_schema()
        validate_schema(json.loads(self.json()), schema)

        if remote:
            # Ignore output ports as gbdxtools will override value.
            ports = [
                port for port in self.input_ports if port.port_type == 'directory'
                ]
            for port in ports:
                # Will raise exception if the port is invalid.
                port.is_valid_s3_url(port.value)
        else:
            all_ports = self.ports[0] + self.ports[1]
            ports = [
                port for port in all_ports if port.port_type == 'directory' and port.name != 'source_bundle'
                ]
            for port in ports:
                # Will raise exception if the port is invalid.
                port.is_valid_filesys(port.value)

        return True


class CustomEncoder(json.JSONEncoder):
    def default(self, obj):
        # For Port instances
        if isinstance(obj, Port):
            return obj.json
        # Let the base class default method raise the TypeError
        return json.JSONEncoder.default(self, obj)



import os
import pytest
import vcr

from gbdx_cloud_harness.test.auth_mock import get_mock_gbdx_session
from gbdx_task_template import TaskTemplate, Task, InputPort, OutputPort


# So vcr doesn't record Amazon requests, moto will mock this.
my_vcr = vcr.VCR(
    ignore_hosts=['s3.amazonaws.com', '169.254.169.254'],  # This IP address is a temp fix.
)


@pytest.fixture(scope='module')
def mock_auth():
    return get_mock_gbdx_session()


@pytest.fixture(scope='module')
def test_path():
    return os.path.dirname(os.path.abspath(__file__))


# Test class
class BasicApp(TaskTemplate):

    task = Task("MyCustomTask")  # Create the Task
    task.properties = {
        "timeout": 9600,
        "isPublic": False
    }  # Update Properties

    task.first_port = InputPort(port_type="string", value="Hello")
    task.second_port = InputPort(value="~/mydata/input/")

    task.output_port = OutputPort(value="~/mydata/output/")

    def invoke(self):
        print("\n\tHello, World!")


@pytest.fixture()
def TestApp():
    return BasicApp







from unittest import TestCase
import os
import mock

from gbdx_task_template import TaskTemplate, Task, InputPort, OutputPort


class MyBasicApp(TaskTemplate):

    task = Task("MyCustomTask")  # Create the Task
    task.properties = {
        "timeout": 9600,
        "isPublic": False
    }  # Update Properties

    task.first_port = InputPort(port_type="string", value="Hello")
    task.second_port = InputPort(value="~/mydata/input/")

    task.output_port = OutputPort(value="~/mydata/output/")

    def invoke(self):
        print("\n\tHello, World!")


MY_BASIC_APP_WF_DEF = {
    'tasks': [
        {
            'outputs': [
                {
                    'name': 'output_port'
                }
            ],
            'containerDescriptors': [
                {
                    'properties': {
                        'domain': 'default'
                    }
                }
            ],
            'taskType': 'MyCustomTask',
            'name': 'MyCustomTask',
            'inputs': [
                {
                    'name': 'second_port',
                    'value': '~/mydata/input/'
                },
                {
                    'name': 'first_port',
                    'value': 'Hello'
                },
                {
                    'name': 'source_bundle',
                    'value': '/Users/michaelconnor/Documents/SparkGeo/projects/Digital_Globe/TDGP/cloud-harness/source_bundle'
                }
            ]
        },
        {
            'outputs': [],
            'containerDescriptors': [
                {
                    'properties': {
                        'domain': 'default'
                    }
                }
            ],
            'taskType': 'StageDataToS3',
            'name': 'StageDataToS3_ed92cb3f-6b92-42a6-9776-202c0fa3c80e',
            'inputs': [
                {
                    'name': 'destination',
                    'value': 's3://gbd-customer-data/a157fdce-bb1d-42b3-96a9-66942896a787/e3c3bc3f-d406-4bd2-821d-a73a8badc9f5/MyCustomTask/output_port'
                },
                {
                    'source': 'MyCustomTask_7562bdd4-4cc7-4dcb-b3a7-ad6dd1f7bead:output_port',
                    'name': 'data'
                }
            ]
        }
    ],
    'name': 'e3c3bc3f-d406-4bd2-821d-a73a8badc9f5'
}


class BadTaskApp(TaskTemplate):
    task = Task("BadTask")  # Create the Task

    # No Input ports, bad schema.

    task.output_port = OutputPort(value="~/mydata/output/")

    def invoke(self):
        print("\n\tHello, World!")


class TestBase(TestCase):

    def setUp(self):
        self.task = Task("MyCustomTask")
        self.task.properties = {
            "timeout": 9600,
            "isPublic": False
        }  # Update Properties

        test_path = os.path.abspath(os.path.dirname(__file__))

        self.task.first_port = InputPort(port_type="string", value="Hello")
        self.task.second_port = InputPort(value=test_path)

        self.task.output_port = OutputPort(value=test_path)

        self.basic_inport_names = ['source_bundle', 'first_port', 'second_port']
        self.basic_outport_names = ['output_port']


def temp_env_var(**kwargs):
    """
    Decorator used for temporarily patching environment
    variables.
    :param func: The function to patch the env vars for.
    :param kwargs: key word args for env vars
    :return:
    """
    def wrap(func):

        def wrapped(self):
            """
            Temporarily set env vars from kwargs.
            :return:
            """
            with mock.patch.dict('os.environ', **kwargs):
                # Call the function
                func(self)
        return wrapped
    return wrap



from setuptools import setup, find_packages


setup(
    name='gbdx-cloud-harness',
    version='0.2.9',
    packages=find_packages(exclude=['docs', 'examples', 'docker_runner']),
    include_package_data=True,
    install_requires=[
        'boto3>=1.3.0',
        'docopt>=0.6.2',
        'jsonschema==2.4.0',
        'future>=0.15.2'
    ],
    url='https://github.com/TDG-Platform/cloud-harness',
    license='MIT',
    description='GBDX Cloud Harness',
    long_description=open('README.rst').read(),
    author='GBDX (Michael Connor)',
    author_email='mike@sparkgeo.com',
    entry_points={
        'console_scripts': ['cloud-harness=gbdx_cloud_harness.controller:main'],
    },
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2.7',
    ],
    tests_require=['pytest', 'vcrpy', 'moto', 'coverage']
)



import os
import boto3
from boto3.s3.transfer import S3Transfer

from gbdx_auth import gbdx_auth
from gbdx_cloud_harness.utils.printer import printer


class AccountStorageService(object):
    """
    Provides an interface to a Users Account Storage (S3). To be used when accessing
    the users account storage. All access is through the GBDX temporary credentials service.
    """

    def __init__(self, is_test=False):
        """
        Constructor for AccountStorageService class
        :param session: an instance of gbdx_auth (requests.OAuth2)
        """

        endpoint = 'https://geobigdata.io/s3creds/v1/prefix'

        if not is_test:
            self.gbdx = gbdx_auth.get_session()

            response = self.gbdx.get('%s?duration=3600' % endpoint)

            if response.status_code != 200:
                raise Exception('%s: %s' % (response.status_code, response.content))

            secret_key = response.json()['S3_secret_key']
            access_key = response.json()['S3_access_key']
            session_token = response.json()['S3_session_token']

            session = boto3.session.Session(
                aws_access_key_id=access_key,
                aws_secret_access_key=secret_key,
                aws_session_token=session_token
            )

            self.bucket = response.json()['bucket']
            self.prefix = response.json()['prefix']

            self.client = session.client('s3')

        else:
            # session is not None, meaning it is being overridden.
            # Testing purposes.

            self.bucket = "not_provided"
            self.prefix = "not_provided"

            # Session creds don't matter because of the mock_s3 decorator
            session = boto3.session.Session(
                region_name='us-east-1'
            )
            self.client = session.client('s3')
            self.client.create_bucket(Bucket=self.bucket)

        self.s3 = S3Transfer(self.client)

    @property
    def location(self):
        return 's3://%s/%s' % (self.bucket, self.prefix)

    @property
    def auth(self):
        return self.gbdx

    def upload(self, source_files, s3_folder=None):
        """
        Upload a list of files to a users account location
        :param source_files: list of files to upload, or single file name
        :param s3_folder: the user location to upload to.
        """

        if s3_folder is None:
            folder = self.prefix
        else:
            folder = '%s/%s' % (self.prefix, s3_folder)

        if isinstance(source_files, list):
            for file_tuple in source_files:
                self.__upload_file(file_tuple, folder)
        elif isinstance(source_files, tuple):
            self.__upload_file(source_files, folder)
        else:
            raise ValueError("Source Files must be a tuple or list of tuples: (filename, keyname)")

    def __upload_file(self, file_tuple, folder):
        key_name = '%s/%s' % (folder, file_tuple[1])
        printer('%s uploaded to location %s' % (file_tuple[0], file_tuple[1]))
        self.s3.upload_file(file_tuple[0], self.bucket, key_name)

    def download(self, local_port_path, key_names):
        """
        download all files from a users account location
        :param local_port_path: the local path where the data is to download to
        :param key_name: can start with self.prefix or taken as relative to prefix.

        Example:
            local_port_path = /home/user/myworkflow/input_images/ (sync all data in this folder)
            s3_folder = myworkflow/input_images/ (location on s3 that will be synced to local path)
        """

        if not os.path.isdir(local_port_path):
            raise ValueError("Download path does not exist: %s" % local_port_path)

        if not isinstance(key_names, list):
            key_names = [key_names]

        for key_name in key_names:
            is_folder = key_name.endswith('/')

            # strip leading and trailing slashes
            key_name = key_name.lstrip('/').rstrip('/')
            key_parts = key_name.split('/')

            # Key names from the list function will include the account prefix
            # and any folder namespace.
            if key_parts[0] == self.prefix:
                path = os.path.join(local_port_path, *key_parts[1:])
                if not is_folder:
                    folder_path = os.path.join(local_port_path, *key_parts[1:-1])
                get_key_name = key_name
            else:
                path = os.path.join(local_port_path, *key_parts)
                if not is_folder:
                    folder_path = os.path.join(local_port_path, *key_parts[:-1])
                get_key_name = '%s/%s' % (self.prefix, key_name)

            if is_folder and not os.path.isdir(path):
                # A directory that doesn't exist
                os.makedirs(path)
            else:
                if not os.path.isdir(folder_path):
                    os.makedirs(folder_path)
                # Assume it is a file
                self.__download_file(path, get_key_name)

    def __download_file(self, filename, key_name):
        self.client.download_file(self.bucket, key_name, filename)

    def list(self, s3_folder='', full_key_data=False):
        """Get a list of keys for the accounts"""
        if not s3_folder.startswith('/'):
            s3_folder = '/' + s3_folder

        s3_prefix = self.prefix + s3_folder

        bucket_data = self.client.list_objects(Bucket=self.bucket, Prefix=s3_prefix)

        if full_key_data:
            return bucket_data['Contents']
        else:
            return [k['Key'] for k in bucket_data['Contents']]
